@@ -315,6 +315,27 @@ class AlignmentDepictionTools(AlignmentBackEndEditingTools):
         if alignLength < 1:
             return
         #
+        found_non_ala_residue = False
+        found_ala_residue = False
+        for alignTupList in self._seqAlignList:
+            for alignTup in alignTupList:
+                if (alignTup[1] == self._gapSymbol) or (alignTup[1].upper() == "ALA"):
+                    if alignTup[1].upper() == "ALA":
+                        found_ala_residue = True
+                    #
+                    continue
+                #
+                found_non_ala_residue = True
+                break
+            #
+            if found_non_ala_residue:
+                break
+            #
+        #
+        all_ala_flag = False
+        if (not found_non_ala_residue) and found_ala_residue:
+            all_ala_flag = True
+        #
         fp = open(htmlFilePath, "w")
         #
         alignmentLine = 0
@@ -432,6 +453,9 @@ class AlignmentDepictionTools(AlignmentBackEndEditingTools):
                         ii = self._seqAlignList[sPos][alignIdx][5].find("hetero")
                         idS += "_" + self._seqAlignList[sPos][alignIdx][5][ii + 7 :]
                     #
+                    if all_ala_flag and (seqType == "auth"):
+                        cssPosClassBg += " cf-ala-unk "
+                    #
                     # Add highlighting for linker regions in the sample sequence --
                     #
                     if (seqType == "auth") and (len(self._seqAlignList[sPos][alignIdx][5]) > 0):
@@ -503,6 +527,8 @@ class AlignmentDepictionTools(AlignmentBackEndEditingTools):
         self.__annotationDict["mutationOrig"] = featureObj.getEntityMutationDetailsOrig()
         self.__annotationDict["description"] = featureObj.getEntityDescription()
         self.__annotationDict["descriptionOrig"] = featureObj.getEntityDescriptionOrig()
+        self.__annotationDict["details"] = featureObj.getEntityDetails()
+        self.__annotationDict["detailsOrig"] = featureObj.getEntityDetailsOrig()
         self.__annotationDict["source_method"] = featureObj.getEntityDescriptionOrig()
         #
         fp = open(htmlFilePath, "w")
@@ -516,6 +542,10 @@ class AlignmentDepictionTools(AlignmentBackEndEditingTools):
         fp.write("<td><b>Mutation</b></td>")
         fp.write("<td>%s</td>" % self.__annotationDict["mutation"])
         fp.write("<td>%s</td>" % self.__annotationDict["mutationOrig"])
+        fp.write("</tr><tr>")
+        fp.write("<td><b>Entity details</b></td>")
+        fp.write("<td>%s</td>" % self.__annotationDict["details"])
+        fp.write("<td>%s</td>" % self.__annotationDict["detailsOrig"])
         fp.write("</tr>")
         fp.write("</table>")
         fp.close()
