@@ -18,6 +18,7 @@
 #  01-Oct-2022  zf  modified __getAuthDefinedRefD() method to include author provided "seq_align_begin", "seq_align_end", "db_align_end",
 #                   "db_align_beg", & "db_seq_one_letter_code" information
 #                   improved process to handle the author-provided reference IDs and new entity summary page
+#  06-Jan-2024  zf  using relevant UniProt sequence from Sequence Builder to build the reference sequence information instead of BLAST searching
 ##
 """
 Assemble sequence and other required data to start/restart the sequence editor tool.
@@ -97,7 +98,7 @@ class SequenceDataAssemble(UpdateSequenceDataStoreUtils):
         #
         # minimum sequence length to search in reference databases -
         #
-        self.__minSearchSequenceLengthAA = 12
+        self.__minSearchSequenceLengthAA = 9
         self.__minSearchSequenceLengthNA = 50
         self.__entityIdList = []
         #
@@ -112,7 +113,8 @@ class SequenceDataAssemble(UpdateSequenceDataStoreUtils):
         self.__doRefSearchFlag = doRefSearch
         #
         if self._verbose:
-            self._lfh.write("\n+SequenceDataAssemble.doAssemble() STARTING with depId %r calPolyLink %r doRefSearch %r\n" % (self.__dataSetId, calPolyLink, doRefSearch))
+            self._lfh.write("\n+SequenceDataAssemble.doAssemble() STARTING with depId %r calPolyLink %r doRefSearch %r doAutoProcess %r\n" %
+                           (self.__dataSetId, calPolyLink, doRefSearch, doAutoProcess))
         #
         pdbxFilePath = self.__pI.getModelPdbxFilePath(self.__dataSetId, fileSource="session", versionId="1")
         polyLinkPath = self.__pI.getFilePath(self.__dataSetId, contentType="polymer-linkage-distances", formatType="json", fileSource="session")
@@ -125,7 +127,9 @@ class SequenceDataAssemble(UpdateSequenceDataStoreUtils):
         if not os.access(polyLinkPath, os.R_OK):
             return False
         #
-        ok, no_coord_issue, seqSimilarityInfoList, polymerLinkDistList, entryD, entityD, instanceD, statisticsMap, depSeqAssign, seqAssign = self.__readJsonObject(polyLinkPath)
+        ok, no_coord_issue, seqSimilarityInfoList, polymerLinkDistList, missingResidueInfoList, mixMseMetResidueList, entryD, entityD, instanceD, statisticsMap, depSeqAssign, \
+            seqAssign = self.__readJsonObject(polyLinkPath)
+        #
         if not ok:
             return False
         #
@@ -178,9 +182,8 @@ class SequenceDataAssemble(UpdateSequenceDataStoreUtils):
         except:  # noqa: E722 pylint: disable=bare-except
             sortedEntityIdList = self.__entityIdList
         #
-        self.__createSeqMatchFileAndMisMatchPickleFile(
-            sortedEntityIdList, entityD, ownRefD, eSSRefD, list(newOldEntityIdMap.keys()), misMatchList, notFoundMatchList, seqSimilarityInfoList
-        )
+        self.__createSeqMatchFileAndMisMatchPickleFile(sortedEntityIdList, entityD, ownRefD, eSSRefD, list(newOldEntityIdMap.keys()), misMatchList,
+                                                       notFoundMatchList, seqSimilarityInfoList, missingResidueInfoList, mixMseMetResidueList)
         #
         self.__updateUtilDataStore(pdbxFilePath)
         #
@@ -279,9 +282,14 @@ class SequenceDataAssemble(UpdateSequenceDataStoreUtils):
         #
         seqSearchOp = str(self._reqObj.getValue("seq_search_op"))
         if seqSearchOp == "on":
+            flag = False
+            withRefInfo = str(self._reqObj.getValue("withref_info"))
+            if withRefInfo == "yes":
+                flag = True
+            #
             self.__minSearchSequenceLengthAA = 5
             self.__minSearchSequenceLengthNA = 15
-            self.__reRunSearch([entityId])
+            self.__reRunSearch([entityId], withRefFlag=flag)
         #
         self.__updateSelections([entityId])
         self.saveSequenceDataStore()
@@ -325,6 +333,8 @@ class SequenceDataAssemble(UpdateSequenceDataStoreUtils):
             no_coord_issue = True
             seqSimilarityInfoList = []
             polymerLinkDistList = []
+            missingResidueInfoList = []
+            mixMseMetResidueList = []
             entryD = {}
             entityD = {}
             instanceD = {}
@@ -357,18 +367,63 @@ class SequenceDataAssemble(UpdateSequenceDataStoreUtils):
                 #
                 if ("depSeqAssign" in jsonObj) and jsonObj["depSeqAssign"]:
                     depSeqAssign = jsonObj["depSeqAssign"]
+                    for key, valD in depSeqAssign.items():
+                        if ("ref_list" not in valD) or (not valD["ref_list"]):
+                            continue
+                        #
+                        for refD in valD["ref_list"]:
+                            for item in ("db_name", "db_accession", "db_code", "seq_align_begin", "seq_align_end", "db_align_end",
+                                         "db_align_beg", "db_seq_one_letter_code"):
+                                if (item in refD) and refD[item]:
+                                    refD[item] = refD[item].strip().upper()
+                                #
+                            #
+                        #
+                    #
                 #
                 if ("annSeqAssign" in jsonObj) and jsonObj["annSeqAssign"]:
                     seqAssign = jsonObj["annSeqAssign"]
                 #
             #
-            return True, no_coord_issue, seqSimilarityInfoList, polymerLinkDistList, entryD, entityD, instanceD, statisticsMap, depSeqAssign, seqAssign
+            for key, valD in entityD.items():
+                if ("SEQ_TUP_LIST" not in valD) or (not valD["SEQ_TUP_LIST"]):
+                    continue
+                #
+                hasMSE = False
+                hasMET = False
+                for seqTupl in valD["SEQ_TUP_LIST"]:
+                    if seqTupl[0] == "MSE":
+                        hasMSE = True
+                    elif seqTupl[0] == "MET":
+                        hasMET = True
+                    #
+                #
+                if hasMSE and hasMET:
+                    mixMseMetResidueList.append("Entity " + key + " contains both MET and MSE.")
+                #
+                if ("INSTANCE_LIST" not in valD) or (not valD["INSTANCE_LIST"]):
+                    continue
+                #
+                for instId in valD["INSTANCE_LIST"]:
+                    if instId not in instanceD:
+                        continue
+                    #
+                    if (10 * len(instanceD[instId])) > (7 * len(valD["SEQ_TUP_LIST"])):
+                        continue
+                    #
+                    percent = float((len(valD["SEQ_TUP_LIST"]) - len(instanceD[instId])) * 100) / float(len(valD["SEQ_TUP_LIST"]))
+                    missingResidueInfoList.append("%.1f" % percent + "% residues of chain '" + instId + "' (%d/%d residues) are missing in coordinates."
+                                                  % (len(valD["SEQ_TUP_LIST"]) - len(instanceD[instId]), len(valD["SEQ_TUP_LIST"])))
+                #
+            #
+            return True, no_coord_issue, seqSimilarityInfoList, polymerLinkDistList, missingResidueInfoList, mixMseMetResidueList, entryD, entityD, instanceD, \
+                statisticsMap, depSeqAssign, seqAssign
         except:  # noqa: E722 pylint: disable=bare-except
             if self._verbose:
                 traceback.print_exc(file=self._lfh)
             #
         #
-        return False, False, [], [], {}, {}, {}, {}, {}, {}
+        return False, False, [], [], [], [], {}, {}, {}, {}, {}, {}
 
     def __getAuthDefinedRefD(self, depSeqAssign):
         """Get author provided reference information from pdbx_struct_ref_seq_depositor_info category"""
@@ -485,9 +540,9 @@ class SequenceDataAssemble(UpdateSequenceDataStoreUtils):
         tmpPolyLinkPath = os.path.join(self.__sessionPath, self.__dataSetId + "-poly-link.json")
         self.__calcPolymerLinkages(pdbxFilePath=prevModelFilePath, polyLinkPath=tmpPolyLinkPath)
         #
-        ok, _no_coord_issue, _seqSimilarityInfoList, _polymerLinkDistList, _entryD, prevEntityD, prevInstanceD, _statisticsMap, _depSeqAssign, _seqAssign = self.__readJsonObject(
-            tmpPolyLinkPath
-        )
+        ok, _no_coord_issue, _seqSimilarityInfoList, _polymerLinkDistList, _missingResidueInfoList, _mixMseMetResidueList, _entryD, prevEntityD, prevInstanceD, _statisticsMap, \
+            _depSeqAssign, _seqAssign = self.__readJsonObject(tmpPolyLinkPath)
+        #
         if not ok:
             return {}, [], []
         #
@@ -606,7 +661,7 @@ class SequenceDataAssemble(UpdateSequenceDataStoreUtils):
         #
         return True
 
-    def __doReferenceSearch(self, entityD, entityIdList):
+    def __doReferenceSearch(self, entityD, entityIdList, withRefFlag=False):
         """Perform the reference sequence database search using the input entity dictionary.
         Store matching results in the local session directory.
         """
@@ -633,6 +688,19 @@ class SequenceDataAssemble(UpdateSequenceDataStoreUtils):
                 if polyTypeCode == "DNA":
                     NA_flag = True
                 #
+                if skip and (polyTypeCode == "AA"):
+                    oneLetterCodeSeq = str(eD["SEQ_ENTITY_1_CAN"]).strip().upper()
+                    countALA = 0
+                    for oneLetterCode in oneLetterCodeSeq:
+                        if oneLetterCode == "A":
+                            countALA += 1
+                        #
+                    #
+                    # Stop autoProcess for poly-ALA Seq per ticket# DAOTHER-8256
+                    if (10 * countALA) > (9 * seqLen):
+                        self.__autoProcessFlag = False
+                    #
+                #
                 if self._verbose:
                     self._lfh.write("+SequenceDataAssemble.__doReferenceSearch() search for entity id %s type %s length %d skip status %r\n" % (eId, polyTypeCode, seqLen, skip))
                     self._lfh.flush()
@@ -653,7 +721,13 @@ class SequenceDataAssemble(UpdateSequenceDataStoreUtils):
                 #
                 return {}, {}, {}
             #
-            # refD,ownRefD,otherRefD = self.__runSeqAllSearches(entityTupList)
+            if withRefFlag and (len(entityTupList) == 1):
+                seqSearchUtil = LocalBlastSearchUtils(siteId=self.__siteId, sessionPath=self.__sessionPath, pathInfo=self.__pI,
+                                                      doRefSearchFlag=self.__doRefSearchFlag, verbose=self._verbose, log=self._lfh)
+                predefinedRefD = seqSearchUtil.getPredefinedRefSequence(dataSetId=self.__dataSetId, entityD=entityTupList[0][1])
+                if predefinedRefD:
+                    return predefinedRefD, {}, {}
+                #
             #
             ownRefD, otherRefD = self.__runSameSeqAnnotationSearch(entityTupList)
             #
@@ -736,82 +810,6 @@ class SequenceDataAssemble(UpdateSequenceDataStoreUtils):
         #
         return {}, {}, {}
 
-    # def __runSeqAllSearches(self, entityTupList):
-    #     """Perform all searches of the reference sequences"""
-    #     try:
-    #         startTime = time.time()
-    #         #
-    #         if self.__cI.get("USE_COMPUTE_CLUSTER"):
-    #             numProc = len(entityTupList)
-    #         else:
-    #             numProc = int(multiprocessing.cpu_count() / 2)
-    #         mpu = MultiProcUtil(verbose=True)
-    #         mpu.set(workerObj=self, workerMethod="runMultiReferenceSearches")
-    #         mpu.setWorkingDir(self.__sessionPath)
-    #         #
-    #         # Setup limits for NCBI requets
-    #         apikey = self.__cI.get("NCBI_API_KEY")
-    #         apirate = self.__cI.get("NCBI_API_RATE")
-    #         if apikey:
-    #             if apirate:
-    #                 rate = int(apirate)
-    #             else:
-    #                 rate = 5
-    #             #
-    #         else:
-    #             rate = 1
-    #         #
-    #         mpl = MultiProcLimit(rate)
-    #         mpu.setOptions({"ncbilock": mpl})
-    #         #
-    #         _ok, _failList, retLists, _diagList = mpu.runMulti(dataList=entityTupList, numProc=numProc, numResults=1)
-    #         #
-    #         refD = {}
-    #         ownRefD = {}
-    #         otherRefD = {}
-    #         for tupList in retLists:
-    #             for tup in tupList:
-    #                 if tup[1]:
-    #                     refD[tup[0]] = tup[1]
-    #                 #
-    #                 if tup[2]:
-    #                     ownRefD[tup[0]] = tup[2]
-    #                 #
-    #                 if tup[3]:
-    #                     otherRefD[tup[0]] = tup[3]
-    #                 #
-    #             #
-    #         #
-    #         if self._verbose:
-    #             endTime = time.time()
-    #             self._lfh.write(
-    #                 "+SequenceDataAssemble.__runSeqAllSearches()  completed at %s (%.2f seconds)\n" % (time.strftime("%Y %m %d %H:%M:%S", time.localtime()), endTime - startTime)
-    #             )
-    #         #
-    #         return refD, ownRefD, otherRefD
-    #     except:  # noqa: E722 pylint: disable=bare-except
-    #         if self._verbose:
-    #             self._lfh.write("+SequenceDataAssemble.__runSeqAllSearches() - failing \n")
-    #             traceback.print_exc(file=self._lfh)
-    #         #
-    #     #
-    #     return {}, {}, {}
-
-    # EP: Believe unused
-#   def runMultiReferenceSearches(self, dataList, procName, optionsD, workingDir):  # pylint: disable=unused-argument
-#       """Multiple reference sequence search processing API"""
-#       ncbilock = optionsD.get("ncbilock", None)
-#       rList = []
-#       for tupL in dataList:
-#           seqSearchUtil = SeqReferenceSearchUtils(
-#               siteId=self.__siteId, sessionPath=self.__sessionPath, pathInfo=self.__pI, verbose=self._verbose, log=self._lfh, ncbilock=ncbilock
-#           )
-#           eRefD, selfRefD, sameSeqRefD = seqSearchUtil.run(self.__dataSetId, tupL[1], tupL[2], self.__doRefSearchFlag, self.__forceBlastSearchFlag)
-#           rList.append((tupL[0], eRefD, selfRefD, sameSeqRefD))
-#           #
-#       #
-#       return rList, rList, []
-
     def __runSameSeqAnnotationSearch(self, entityTupList):
         """Search same sequence annotation information from processed entries"""
         try:
@@ -947,9 +945,8 @@ class SequenceDataAssemble(UpdateSequenceDataStoreUtils):
         #
         return rList, rList, []
 
-    def __createSeqMatchFileAndMisMatchPickleFile(
-        self, sortedEntityIdList, entityD, ownRefD, eSSRefD, prevEntityIdList, prevMisMatchList, prevNotFoundMatchList, seqSimilarityInfoList
-    ):
+    def __createSeqMatchFileAndMisMatchPickleFile(self, sortedEntityIdList, entityD, ownRefD, eSSRefD, prevEntityIdList, prevMisMatchList,
+                                                  prevNotFoundMatchList, seqSimilarityInfoList, missingResidueInfoList, mixMseMetResidueList):
         """Create D_xxxxxxxxxx_seqmatch_P1.cif.Vx file"""
         annList = []
         misMatchList = []
@@ -1022,6 +1019,8 @@ class SequenceDataAssemble(UpdateSequenceDataStoreUtils):
         #
         warningMsgMap["mismatch"] = misMatchList
         warningMsgMap["not_found_existing_match"] = notFoundMatchList
+        warningMsgMap["missing_residue"] = missingResidueInfoList
+        warningMsgMap["mix_mse_met"] = mixMseMetResidueList
         #
         fb = open(self.__misMatchPath, "wb")
         pickle.dump(warningMsgMap, fb)
@@ -1139,7 +1138,7 @@ class SequenceDataAssemble(UpdateSequenceDataStoreUtils):
             alignUtil.setInputAlignData(alignD=dataD)
         #
 
-    def __reRunSearch(self, selectedEntityIdList):
+    def __reRunSearch(self, selectedEntityIdList, withRefFlag=False):
         """ """
         self.__doRefSearchFlag = True
         self.__forceBlastSearchFlag = True
@@ -1160,7 +1159,7 @@ class SequenceDataAssemble(UpdateSequenceDataStoreUtils):
                 self.__setEntityAlignInfoMap(authLabel, eD)
             #
         #
-        eRefD, ownRefD, eSSRefD = self.__doReferenceSearch(entityD, list(entityD.keys()))
+        eRefD, ownRefD, eSSRefD = self.__doReferenceSearch(entityD, list(entityD.keys()), withRefFlag=withRefFlag)
         self.setMultipleEntitiesRefSequenceInfo(entityD, eRefD, ownRefD, eSSRefD)
         #
         self.__resetAlignmentFlag = True
