@@ -19,7 +19,7 @@ import sys
 import traceback
 
 
-class BlastPlusReader:
+class BlastPlusReader(object):
     """Read Blast+ result file (xml format)  and return the list of dictionaries containing the following --
 
     dict['db_name']
@@ -38,6 +38,8 @@ class BlastPlusReader:
     dict['hitTo']
     dict['alignLen']
     dict['match_length']
+    dict['query_length']
+    dict['db_length']
 
     """
 
