@@ -6,6 +6,7 @@
 #  3-Oct-2022  zf  update fetchReferenceSequenceWithSeqMatch() methods for better handling author provided reference sequence cases.
 #                  add runSeqAlignment() & __toList() methods
 # 11-Nov-2022  zf  move __getRefInfo() method to FetchSeqInfoUtils class.
+#  6-Jan-2024  zf  add reset() method
 ##
 """
 Methods to get reference sequence data from reference database based database name and identifier.
@@ -36,7 +37,12 @@ class FetchReferenceSequenceUtils(object):
         self.__srd = seqReferenceData
         if not self.__srd:
             self.__srd = SequenceReferenceData(self.__verbose, self.__lfh)
+        self.__accCode = ""
+        self.__refInfoD = {}
         #
+        self.reset()
+
+    def reset(self):
         self.__accCode = ""
         self.__refInfoD = {}
 
