@@ -19,6 +19,7 @@
 #  8-Dec-2015  jdw change isoform annoation processing -
 #  3-Oct-2022  zf  update __runBlastSearch() & __fetchAuthProvidedRefSequence() methods for better handling author provided
 #                  reference sequence cases.
+#  6-Jan-2024  zf  add getPredefinedRefSequence(), __readPredefinedRefSequenceInfo() and __processPredefinedRefSequence() methods
 ##
 """
 Methods to manage local and sequence search services and package matching reference sequence data.
@@ -36,6 +37,7 @@ except ImportError:
     import pickle as pickle
 #
 import multiprocessing
+import copy
 import os
 import sys
 import time
@@ -49,6 +51,7 @@ from wwpdb.apps.seqmodule.util.FetchReferenceSequenceUtils import FetchReference
 from wwpdb.apps.seqmodule.util.SequenceReferenceData import SequenceReferenceData
 from rcsb.utils.multiproc.MultiProcUtil import MultiProcUtil
 from wwpdb.io.locator.PathInfo import PathInfo
+from wwpdb.utils.config.ConfigInfoApp import ConfigInfoAppCommon
 from wwpdb.utils.dp.RcsbDpUtility import RcsbDpUtility
 
 
@@ -65,7 +68,7 @@ class LocalBlastSearchUtils(object):
         self.__ncbilock = ncbilock
         #
         self.__minRnaSequenceSearchLength = 50
-        self.__maxHitsSearch = 100
+        self.__maxHitsSearch = 250
         self.__maxHitsSave = 50
         self.__shortSequenceLengthLimit = 20
         self.__cleanUp = True
@@ -87,6 +90,9 @@ class LocalBlastSearchUtils(object):
         self.__taxUtils = TaxonomyUtils(siteId=self.__siteId, verbose=self.__verbose, log=self.__lfh)
         self.__srd = SequenceReferenceData(self.__verbose, self.__lfh)
         #
+        self.__cICommon = ConfigInfoAppCommon(self.__siteId)
+        self.__seqDbPath = self.__cICommon.get_site_refdata_sequence_db_path()
+        #
         self.__matchEntityAttribNameList = [
             "id",
             "fragment_id",
@@ -150,6 +156,25 @@ class LocalBlastSearchUtils(object):
         #
         return autoMatchStatus, rD
 
+    def getPredefinedRefSequence(self, dataSetId=None, entityD=None):
+        """ Get reference sequence information for the sequence builder application
+        """
+        self.__dataSetId = dataSetId
+        self.__entityD = entityD
+        if not self.__entityD:
+            return {}
+        #
+        oneLetterCodeSeq = str(self.__entityD["SEQ_ENTITY_1_CAN"]).strip().upper()
+        self.__fullOneLetterSeq = self.__srd.toList(oneLetterCodeSeq)
+        if not self.__checkPartRange(len(self.__fullOneLetterSeq), self.__entityD["PART_LIST"]):
+            return {}
+        #
+        refSeqInfoD, matchedRefList = self.__readPredefinedRefSequenceInfo()
+        if (not refSeqInfoD) or (not matchedRefList):
+            return {}
+        #
+        return self.__processPredefinedRefSequence(oneLetterCodeSeq, refSeqInfoD, matchedRefList)
+
     def __getExistingSearchResult(self):
         """Get prvious blast search result"""
         self.__entityId = self.__entityD["ENTITY_ID"]
@@ -227,6 +252,10 @@ class LocalBlastSearchUtils(object):
         #
         if "SEQ_ENTITY_1_SEARCH" in self.__entityD:
             oneLetterSearchSeq = str(self.__entityD["SEQ_ENTITY_1_SEARCH"]).strip().upper()
+            # Use one letter code 'X' instead of multiple one letter codes defined in CCD for chromophore residue (DAOTHER-9059)
+            if len(oneLetterSearchSeq) != len(oneLetterCodeSeq):
+                oneLetterSearchSeq = oneLetterCodeSeq
+            #
         else:
             # for backward compatible
             oneLetterSearchSeq = oneLetterCodeSeq
@@ -282,16 +311,29 @@ class LocalBlastSearchUtils(object):
                                      (seqPartId, seqPartType, seqNumBeg, seqNumEnd))
                     self.__lfh.flush()
                 #
+                if self.__seqType.startswith("polypeptide") and ((10 * countALA) > (9 * countTotal)):
+                    foundPerfectMatch = False
+                #
                 continue
             #
             taxId = fD["SOURCE_TAXID"]
             searchSequence = oneLetterSearchSeq[(int(seqNumBeg) - 1) : int(seqNumEnd)]
             #
+            searchResultFileList = []
             xmlFilePath = os.path.join(self.__sessionPath, self.__dataSetId + "_blast-match_E" + self.__entityId + "_P" + str(partNum) + ".xml")
             if os.access(xmlFilePath, os.F_OK):
                 os.remove(xmlFilePath)
             #
-            partDataList.append((str(partNum), searchSequence, seqNumBeg, seqNumEnd, taxId, xmlFilePath))
+            searchResultFileList.append(xmlFilePath)
+            if self.__seqType in ["polypeptide(L)", "polypeptide(D)", "polypeptide"]:
+                xmlFilePath = os.path.join(self.__sessionPath, self.__dataSetId + "_blast-match_E" + self.__entityId + "_P" + str(partNum) + ".1.xml")
+                #
+                if os.access(xmlFilePath, os.F_OK):
+                    os.remove(xmlFilePath)
+                #
+                searchResultFileList.append(xmlFilePath)
+            #
+            partDataList.append((str(partNum), searchSequence, seqNumBeg, seqNumEnd, taxId, searchResultFileList))
             #
             mutationList = []
             if ("MUTATION_DETAILS" in self.__entityD) and self.__entityD["MUTATION_DETAILS"]:
@@ -325,13 +367,13 @@ class LocalBlastSearchUtils(object):
                     continue
                 #
                 # Skip blast search for short sequence
-                if len(searchSequence) <= self.__shortSequenceLengthLimit:
-                    continue
-                #
+#               if len(searchSequence) <= self.__shortSequenceLengthLimit:
+#                   continue
+#               #
             else:
                 foundPerfectMatch = False
             #
-            blastDataList.append((str(partNum), searchSequence, seqNumBeg, seqNumEnd, taxId, xmlFilePath))
+            blastDataList.append((str(partNum), searchSequence, seqNumBeg, seqNumEnd, taxId, searchResultFileList))
         #
         if not partDataList:
             return foundPerfectMatch, authRefD
@@ -348,11 +390,32 @@ class LocalBlastSearchUtils(object):
             #
             id_count = 1
             for partTup in partDataList:
-                hitList = []
-                if os.access(partTup[5], os.F_OK):
-                    hitList = self.__readBlastResultXmlFile(seqNumBeg=partTup[2], seqNumEnd=partTup[3], seqPartId=partTup[0], authTaxId=partTup[4],
-                                                            blastResultXmlPath=partTup[5])
+                mergedhitList = []
+                uniqueIdList = []
+                for resultFile in partTup[5]:
+                    if not os.access(resultFile, os.F_OK):
+                        continue
+                    #
+                    returnHitList = self.__readBlastResultXmlFile(seqNumBeg=partTup[2], seqNumEnd=partTup[3], seqPartId=partTup[0],
+                                                                  blastResultXmlPath=resultFile)
+                    if len(returnHitList) == 0:
+                        continue
+                    #
+                    for hitD in returnHitList:
+                        refID = ""
+                        if ("db_isoform" in hitD) and hitD["db_isoform"]:
+                            refID = hitD["db_isoform"]
+                        elif ("db_accession" in hitD) and hitD["db_accession"]:
+                            refID = hitD["db_accession"]
+                        #
+                        if (refID == "") or (refID in uniqueIdList):
+                            continue
+                        #
+                        uniqueIdList.append(refID)
+                        mergedhitList.append(hitD)
+                    #
                 #
+                hitList = self.__getHitList(mergedhitList, partTup[4])
                 if hitList:
                     SeqAlignmentMap = self.__getRefSeqAlignments(hitList)
                     for hit in hitList:
@@ -459,6 +522,123 @@ class LocalBlastSearchUtils(object):
         pickle.dump(pickleObj, fb)
         fb.close()
 
+    def __readPredefinedRefSequenceInfo(self):
+        """ Read reference sequence information generated in UpdatePolymerEntityPartitions.seqBuilderResponder()
+        """
+        #
+        # Read reference sequence information generated in UpdatePolymerEntityPartitions.seqBuilderResponder()
+        #
+        refSeqInfoD = {}
+        refSeqPickleFile = os.path.join(self.__sessionPath, "fetchedRefSeqs.pic")
+        if os.access(refSeqPickleFile, os.F_OK):
+            try:
+                fb = open(refSeqPickleFile, "rb")
+                refSeqInfoD = pickle.load(fb)
+                fb.close()
+            except:  # noqa: E722 pylint: disable=bare-except
+                refSeqInfoD = {}
+            #
+        #
+        if not refSeqInfoD:
+            return {}, []
+        #
+        # Read matched sequence information generated in UpdatePolymerEntityPartitions.seqBuilderResponder()
+        #
+        matchedRefList = []
+        self.__entityId = self.__entityD["ENTITY_ID"]
+        matchRefSeqPickleFile = os.path.join(self.__sessionPath, "Entity-" + self.__entityId + "-MatchedRefSeqs.pic")
+        if os.access(matchRefSeqPickleFile, os.F_OK):
+            try:
+                fb = open(matchRefSeqPickleFile, "rb")
+                matchedRefList = pickle.load(fb)
+                fb.close()
+            except:  # noqa: E722 pylint: disable=bare-except
+                matchedRefList = []
+            #
+        #
+        return refSeqInfoD, matchedRefList
+
+    def __processPredefinedRefSequence(self, oneLetterCodeSeq, refSeqInfoD, matchedRefList):
+        """ Create Auth/Seq sequences alignment
+        """
+        predefinedRefD = {}
+        for partNum, fD in enumerate(self.__entityD["PART_LIST"], start=1):
+            try:
+                seqNumBeg = int(fD["SEQ_NUM_BEG"])
+                seqNumEnd = int(fD["SEQ_NUM_END"])
+                foundRefFlag = False
+                for matchedRefD in matchedRefList:
+                    if (matchedRefD["queryFrom"] < seqNumBeg) or (matchedRefD["queryTo"] > seqNumEnd):
+                        continue
+                    #
+                    if matchedRefD["db_accession"] not in refSeqInfoD:
+                        continue
+                    #
+                    authSeq = oneLetterCodeSeq[(seqNumBeg - 1) : seqNumEnd]
+                    matchedAuthSeq = authSeq[(matchedRefD["queryFrom"] - seqNumBeg) : (matchedRefD["queryTo"] - seqNumBeg + 1)]
+                    # matchedAuthSeq = oneLetterCodeSeq[(queryFrom - 1) : queryTo]
+                    matchedRefSeq = refSeqInfoD[matchedRefD["db_accession"]]["sequence"][(matchedRefD["hitFrom"] - 1) : matchedRefD["hitTo"]]
+                    if matchedAuthSeq != matchedRefSeq:
+                        return {}
+                    #
+                    hitD = copy.deepcopy(refSeqInfoD[matchedRefD["db_accession"]])
+                    hitD["query_length"] = str(len(authSeq))
+                    hitD["identity"] = str(len(matchedAuthSeq))
+                    hitD["positive"] = str(len(matchedAuthSeq))
+                    hitD["gaps"] = "0"
+                    hitD["query"] = matchedAuthSeq
+                    hitD["queryFrom"] = str(matchedRefD["queryFrom"] - seqNumBeg + 1)
+                    hitD["queryTo"] = str(matchedRefD["queryTo"] - seqNumBeg + 1)
+                    hitD["subject"] = matchedRefSeq
+                    hitD["hitFrom"] = str(matchedRefD["hitFrom"])
+                    hitD["hitTo"] = str(matchedRefD["hitTo"])
+                    hitD["alignLen"] = str(len(matchedAuthSeq))
+                    hitD["match_length"] = str(len(matchedAuthSeq))
+                    hitD["seq_sim"] = 1.0
+                    hitD["sort_metric"] = "404"
+                    hitD["sort_order"] = "1"
+                    hitD["beg_seq_num"] = fD["SEQ_NUM_BEG"]
+                    hitD["end_seq_num"] = fD["SEQ_NUM_END"]
+                    hitD["fragment_id"] = str(partNum)
+                    #
+                    alignIndex, sTup3L, alignLength, seqSim, seqSimWithGaps, error = self.__getSeqAlignIndex(hitD)
+                    if not error:
+                        hitD["alignment"] = alignIndex
+                        hitD["seq_tup_list"] = sTup3L
+                        hitD["statistics"] = (alignLength, seqSim, seqSimWithGaps)
+                        predefinedRefD[str(partNum)] = [hitD]
+                    else:
+                        self.__lfh.write("+LocalBlastSearchUtils.__processPredefinedRefSequence() get sequence alignment index error: %s\n" % error)
+                        return {}
+                    #
+                    foundRefFlag = True
+                    break
+                #
+                if not foundRefFlag:
+                    return {}
+                #
+            except:  # noqa: E722 pylint: disable=bare-except
+                if self.__verbose:
+                    traceback.print_exc(file=self.__lfh)
+                    self.__lfh.write("+LocalBlastSearchUtils.__processPredefinedRefSequence() for entity %r failed\n" % self.__entityId)
+                #
+                return {}
+            #
+        #
+        if predefinedRefD:
+            retD = {}
+            retD[self.__entityId] = predefinedRefD
+            #
+            currSeq = self.__entityD["SEQ_ENTITY_1"]
+            self.__entitySeq = currSeq.replace(" ", "").replace("\n", "").replace("\t", "")
+            self.__blastResultFile = self.__pI.getFilePath(self.__dataSetId, contentType="seqdb-match", formatType="pic",
+                                                           fileSource="session", partNumber=self.__entityId)
+            self.__writeSearchResult(retD)
+            #
+            return retD
+        #
+        return {}
+
     # def __writeBlastSearchResultCifFile(self, rD):
     #     """Write out the current search result mmcif file for debuging purpose"""
     #     #
@@ -512,27 +692,48 @@ class LocalBlastSearchUtils(object):
                 )
                 self.__lfh.write("+LocalBlastSearchUtils.runMultiLocalBlasts() ncbilock %s\n" % ncbilock)
             #
-            self.__runSingleLocalBlast(oneLetterCodeSeq=tupL[1], blastResultXmlPath=tupL[5], partNum=tupL[0], taxId=tupL[4])
+            if self.__seqType in ["polypeptide(L)", "polypeptide(D)", "polypeptide"]:
+                swissprotDBPath = os.path.join(self.__seqDbPath, "my_swissprot_all.pjs")
+                if os.access(swissprotDBPath, os.F_OK):
+                    self.__runSingleLocalBlast(oneLetterCodeSeq=tupL[1], searchResultFile=tupL[5][0], partNum=tupL[0], taxId=tupL[4], dbName="my_swissprot_all")
+                    if (not self.__findPerfectMath(begSeqNum=tupL[2], endSeqNum=tupL[3], partId=tupL[0], taxId=tupL[4], searchResultFile=tupL[5][0])) and \
+                       (len(tupL[1]) > self.__shortSequenceLengthLimit):
+                        # only search whole uniprot for long sequence based on DAOTHER-7190
+                        self.__runSingleLocalBlast(oneLetterCodeSeq=tupL[1], searchResultFile=tupL[5][1], partNum=tupL[0], taxId=tupL[4], dbName="my_uniprot_all")
+                    #
+                else:
+                    self.__runSingleLocalBlast(oneLetterCodeSeq=tupL[1], searchResultFile=tupL[5][0], partNum=tupL[0], taxId=tupL[4], dbName="my_uniprot_all")
+                #
+            else:
+                self.__runSingleLocalBlast(oneLetterCodeSeq=tupL[1], searchResultFile=tupL[5][0], partNum=tupL[0], taxId=tupL[4], dbName="my_ncbi_nt")
+            #
             rList.append(tupL[0])
         #
         return rList, rList, []
 
-    def __runSingleLocalBlast(self, oneLetterCodeSeq, blastResultXmlPath, partNum, taxId):
+    def __runSingleLocalBlast(self, oneLetterCodeSeq, searchResultFile, partNum, taxId, dbName):
         """Internal method to execute the sequence search according to input polymer type."""
         if self.__verbose:
             self.__lfh.write("+LocalBlastSearchUtils.__runSingleLocalBlast() STARTING %s Launch search for %s sequence = %s\n" % (self.__siteId, self.__seqType, oneLetterCodeSeq))
             self.__lfh.flush()
         #
         timeBegin = time.time()
-        resultPath = os.path.abspath(blastResultXmlPath)
+        resultPath = os.path.abspath(searchResultFile)
         tmpPathAbs = os.path.abspath(self.__sessionPath)
         #
         if self.__seqType in ["polypeptide(L)", "polypeptide(D)", "polypeptide"]:
             dp = RcsbDpUtility(tmpPath=tmpPathAbs, siteId=self.__siteId, verbose=True)
             dp.addInput(name="one_letter_code_sequence", value=oneLetterCodeSeq)
-            dp.addInput(name="db_name", value="my_uniprot_all")
+            dp.addInput(name="db_name", value=dbName)
             dp.addInput(name="num_threads", value="4")
             dp.addInput(name="max_hits", value=self.__maxHitsSearch)
+            if len(oneLetterCodeSeq) <= self.__shortSequenceLengthLimit:
+                dp.addInput(name="short_seq", value=True)
+                dp.addInput(name="evalue", value="0.02")
+            #
+            if taxId:
+                dp.addInput(name="tax_id", value=taxId)
+            #
             dp.op("seq-blastp")
             dp.exp(resultPath)
             if self.__cleanUp and not self.__debug:
@@ -547,6 +748,9 @@ class LocalBlastSearchUtils(object):
                 dp.addInput(name="db_name", value="my_ncbi_nt")
                 dp.addInput(name="num_threads", value="4")
                 dp.addInput(name="max_hits", value=self.__maxHitsSearch)
+                if taxId:
+                    dp.addInput(name="tax_id", value=taxId)
+                #
                 dp.op("seq-blastn")
                 dp.exp(resultPath)
                 if self.__cleanUp and not self.__debug:
@@ -563,7 +767,25 @@ class LocalBlastSearchUtils(object):
             self.__lfh.flush()
         #
 
-    def __readBlastResultXmlFile(self, seqNumBeg, seqNumEnd, seqPartId, authTaxId, blastResultXmlPath):
+    def __findPerfectMath(self, begSeqNum="", endSeqNum="", partId="", taxId="", searchResultFile=None):
+        """ Check if there is a perfect match
+        """
+        if (not begSeqNum) or (not endSeqNum) or (not partId) or (not taxId) or (not searchResultFile) or (not os.access(searchResultFile, os.F_OK)):
+            return False
+        #
+        hitList = self.__readBlastResultXmlFile(seqNumBeg=begSeqNum, seqNumEnd=endSeqNum, seqPartId=partId, blastResultXmlPath=searchResultFile)
+        for hitD in hitList:
+            if ("query_length" not in hitD) or (not hitD["query_length"]) or ("identity" not in hitD) or (not hitD["identity"]) or \
+               ("taxonomy_id" not in hitD) or (not hitD["taxonomy_id"]):
+                continue
+            #
+            if (hitD["query_length"] == hitD["identity"]) and (hitD["taxonomy_id"] == taxId):
+                return True
+            #
+        #
+        return False
+
+    def __readBlastResultXmlFile(self, seqNumBeg, seqNumEnd, seqPartId, blastResultXmlPath):
         """Read blast result"""
         hitList = []
         bpr = BlastPlusReader(verbose=self.__verbose, log=self.__lfh)
@@ -701,7 +923,7 @@ class LocalBlastSearchUtils(object):
             hit["end_seq_num"] = seqNumEnd
             hit["fragment_id"] = seqPartId
         #
-        return self.__getHitList(hitList, authTaxId)
+        return hitList
 
     def __getHitList(self, hitList, authTaxId):
         """ """
@@ -732,6 +954,9 @@ class LocalBlastSearchUtils(object):
         if self.__debug:
             self.__lfh.write("+LocalBlastSearchUtils.__sortHitList() length %d authTaxId %s authAncestorD %r\n" % (len(hitList), authTaxId, authAncestorD.items()))
         #
+        highest_identity_score_value = 0.0
+        highest_identity_score_index = 0
+        same_taxid_indice_list = []
         highest_identity_score_with_taxid_match = 0
         lowest_identity_score_with_taxid_match = 101
         lowest_identity_score_with_refid_match = 101
@@ -740,8 +965,14 @@ class LocalBlastSearchUtils(object):
             if "non_isoform_score" not in hit:
                 hit["non_isoform_score"] = 1
             #
+            hit["db_score"] = 0
             if fullScoreFlag or ("identity_score" not in hit):
                 hit["identity_score"] = (float(hit["identity"]) - float(hit["gaps"])) * 100.0 / float(hit["query_length"])
+                hit["sorting_score"] = hit["identity_score"]
+                if hit["identity_score"] > highest_identity_score_value:
+                    highest_identity_score_value = hit["identity_score"]
+                    highest_identity_score_index = i
+                #
                 if self.__debug:
                     self.__lfh.write(
                         "+ReferencSequenceUtils.__sortHitList() i %r  identity %r gaps %r query_length %r  score %r\n"
@@ -750,8 +981,6 @@ class LocalBlastSearchUtils(object):
                     #
                 if hit["db_name"] == "SP":
                     hit["db_score"] = 1
-                else:
-                    hit["db_score"] = 0
                 #
             #
             if self.__authRefIdList and ((hit["db_code"].strip().upper() in self.__authRefIdList) or (hit["db_accession"].strip().upper() in
@@ -775,6 +1004,7 @@ class LocalBlastSearchUtils(object):
             if authAncestorD and targetAncestorD and "id" in authAncestorD and "id" in targetAncestorD:
                 if authAncestorD["id"] == targetAncestorD["id"]:
                     taxidMatch = 3
+                    same_taxid_indice_list.append(i)
                 elif "p_id" in authAncestorD and authAncestorD["p_id"] == targetAncestorD["id"]:
                     taxidMatch = 2
                 elif "p_id" in targetAncestorD and targetAncestorD["p_id"] == authAncestorD["id"]:
@@ -785,6 +1015,11 @@ class LocalBlastSearchUtils(object):
                     taxidMatch = 1
                 #
             #
+            # Add the following modification per request by DAOTHER-8863
+            #
+            if (hit["db_score"] == 1) and (taxidMatch > 0):
+                taxidMatch = 3
+            #
             hit["taxid_match"] = taxidMatch
             if fullScoreFlag and (taxidMatch == 3):
                 if hit["identity_score"] > highest_identity_score_with_taxid_match:
@@ -819,9 +1054,24 @@ class LocalBlastSearchUtils(object):
                 cutoff_identity_score = lowest_identity_score_with_taxid_match - 0.1
             #
         #
+        # Improvement based on entries D_1000212253/5CVM, D_1000236156/6EEB with highest hit has conflicts
+        if (len(same_taxid_indice_list) > 0) and (highest_identity_score_index not in same_taxid_indice_list) and \
+           ("match_length" in hitList[highest_identity_score_index]) and (int(hitList[highest_identity_score_index]["identity"])
+                                                                          < int(hitList[highest_identity_score_index]["match_length"])):
+            for idx in same_taxid_indice_list:
+                if "match_length" not in hitList[idx]:
+                    continue
+                #
+                allowed_diffs = int(int(hitList[idx]["match_length"]) / 100) + 1
+                if ((int(hitList[idx]["match_length"]) - int(hitList[idx]["identity"])) < allowed_diffs) and \
+                   ((highest_identity_score_value - hitList[idx]["identity_score"]) < 1.0):
+                    hitList[idx]["sorting_score"] = highest_identity_score_value
+                #
+            #
+        #
         # The highest score hit is at the bottom of the list.
         #
-        hitList.sort(key=itemgetter("identity_score", "taxid_match", "code_score", "db_score", "non_isoform_score"))
+        hitList.sort(key=itemgetter("sorting_score", "taxid_match", "code_score", "db_score", "non_isoform_score"))
         #
         # reorder list if found author provided reference id
         #
